package apioneshot

import (
	"errors"
	"fmt"

	"git.sr.ht/~mil/mobroute/api/apirtdatabase"
	"git.sr.ht/~mil/mobroute/api/apirtdatabaseq"
	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/api/apirtroute"
	"git.sr.ht/~mil/mobroute/api/apirtschedule"
	"git.sr.ht/~mil/mobroute/api/apirtstops"
	"git.sr.ht/~mil/mobsql"
)

// OneshotRoute performs routing in a one-shot / single function fashion.
// All parameters for the entire request are passed in the OneShotRouteRequest.
func OneshotRoute(r *OneshotRouteRequest) (*apirtroute.RouteResponse, error) {
	if r.RouteParams == nil {
		return nil, errors.New("Required field RouteParams missing")
	} else if mobrouteRuntime, err := apirtinit.RTInitialize(r.MobrouteRuntimeConfig); err != nil {
		return nil, err
	} else if err := dbPrepareFeedIDs(mobrouteRuntime, r.RouteParams.FeedIDs); err != nil {
		return nil, err
	} else {
		return apirtroute.RTRoute(mobrouteRuntime, r.RouteParams)
	}
}

// OneshotStops performs stops query in a one-shot / single function fashion.
// All parameters for the entire request are passed in the OneShotStopsRequest.
func OneshotStops(r *OneshotStopsRequest) (*apirtstops.StopsResponse, error) {
	if r.StopsParams == nil {
		return nil, errors.New("Required field StopsParams missing")
	} else if mobrouteRuntime, err := apirtinit.RTInitialize(r.MobrouteRuntimeConfig); err != nil {
		return nil, err
	} else if err := dbPrepareFeedIDs(mobrouteRuntime, r.StopsParams.FeedIDs); err != nil {
		return nil, err
	} else {
		return apirtstops.RTStops(mobrouteRuntime, r.StopsParams)
	}
}

// OneshotSchedule performs stops query in a one-shot / single function fashion.
// All parameters for the entire request are passed in the OneShotScheduleRequest.
func OneshotSchedule(r *OneshotScheduleRequest) (*apirtschedule.ScheduleResponse, error) {
	if r.ScheduleParams == nil {
		return nil, errors.New("Required field ScheduleParams missing")
	} else if mobrouteRuntime, err := apirtinit.RTInitialize(r.MobrouteRuntimeConfig); err != nil {
		return nil, err
	} else if err := dbPrepareFeedIDs(mobrouteRuntime, r.ScheduleParams.FeedIDs); err != nil {
		return nil, err
	} else {
		return apirtschedule.RTSchedule(mobrouteRuntime, r.ScheduleParams)
	}
}

func OneshotDatabase(r *OneshotDatabaseRequest) (*apirtdatabase.DatabaseResponse, error) {
	if r.DatabaseParams == nil {
		return nil, errors.New("Required field DatabaseParams missing")
	} else if mobrouteRuntime, err := apirtinit.RTInitialize(r.MobrouteRuntimeConfig); err != nil {
		return nil, err
	} else {
		return apirtdatabase.RTDatabase(mobrouteRuntime, r.DatabaseParams)
	}
}

func OneshotDatabaseq(r *OneshotDatabaseqRequest) (*apirtdatabaseq.DatabaseqResponse, error) {
	if r.DatabaseqParams == nil {
		return nil, errors.New("Required field DatabaseqParams missing")
	} else if mobrouteRuntime, err := apirtinit.RTInitialize(r.MobrouteRuntimeConfig); err != nil {
		return nil, err
	} else {
		return apirtdatabaseq.RTDatabaseq(mobrouteRuntime, r.DatabaseqParams)
	}
}

func dbPrepareFeedIDs(mobrouteRuntime *apirtinit.MobrouteRuntime, feedIDs []int) error {
	if _, err := mobsql.FeedLoad(mobrouteRuntime.MobsqlRuntime, feedIDs); err != nil {
		return fmt.Errorf("Failed to load DB via mobsql: %v", err)
	} else if _, err := mobsql.FeedCompute(mobrouteRuntime.MobsqlRuntime, feedIDs); err != nil {
		return fmt.Errorf("Failed to compute DB via mobsql: %v", err)
	} else {
		return nil
	}
}
