// Package apirtinit contains functionality for initializing a Mobroute Runtime.
// The MobrouteRuntime holds the open connection to the local SQLite DB and
// all other library functionality relies on the MobrouteRuntime.
package apirtinit

import (
	"fmt"
	"log"

	"git.sr.ht/~mil/mobroute/dbschemaextra"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"git.sr.ht/~mil/mobsql"
	"github.com/jmoiron/sqlx"
)

// MobsqlRuntimeConfig is just an alias for Mobsql's RuntimeConfig, see
// mobsql.RuntimeConfig for further documentation.
type MobsqlRuntimeConfig = mobsql.RuntimeConfig

// MobrouteRuntime contains the database connection & MobsqlRuntime.
type MobrouteRuntime struct {
	MobsqlRuntime *mobsql.MobsqlRuntime
	DB            *sqlx.DB
}

// MobrouteRuntimeConfig contains the MobsqlRuntimeConfig & potentially
// in the future some other Mobroute-specific properties for the common
// DB connection & Mobroute params.
type MobrouteRuntimeConfig struct {
	MobsqlRuntimeConfig mobsql.RuntimeConfig `json:"mobsql_runtime_config"`
	Logger              *log.Logger          `json:"-"`
	LogInfo             bool                 `json:"log_info"`
	LogWarn             bool                 `json:"log_warn"`
	LogDebug            bool                 `json:"log_debug"`
}

// RTInitialize initializes a MobrouteRuntime which contains a passthrough
// to the MobsqlRuntime & some additional metadata.
func RTInitialize(params *MobrouteRuntimeConfig) (*MobrouteRuntime, error) {
	if params == nil {
		params = &MobrouteRuntimeConfig{}
	}

	// Configure logging
	if params.Logger != nil {
		utillog.Logger = params.Logger
		params.MobsqlRuntimeConfig.Logger = params.Logger
	}
	if params.LogInfo {
		utillog.LogInfo = true
	}
	if params.LogWarn {
		utillog.LogWarn = true
	}
	if params.LogDebug {
		utillog.LogDebug = true
	}
	//log.SetFlags(log.LstdFlags | log.Llongfile)

	// E.g. to enable computed tables etc.
	params.MobsqlRuntimeConfig.SchemaExtra = &dbschemaextra.MobsqlSchemaExtra

	if mobsqlRuntime, err := mobsql.InitializeRuntime(&params.MobsqlRuntimeConfig); err != nil {
		return nil, fmt.Errorf("Error initializing mobsql runtime: %v", err)
	} else {
		return &MobrouteRuntime{
			MobsqlRuntime: mobsqlRuntime,
			DB:            mobsqlRuntime.DB,
		}, nil
	}
}
