package apirtroute

import (
	"fmt"
	"time"

	"git.sr.ht/~mil/mobroute/api/apirtinit"
	"git.sr.ht/~mil/mobroute/api/apirtroute/calcwalk"
	"git.sr.ht/~mil/mobroute/api/apirtroute/diagnosticst"
	"git.sr.ht/~mil/mobroute/csa"
	"git.sr.ht/~mil/mobroute/dbquery"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"github.com/go-playground/validator/v10"
)

// RTRoute performs the routing logic, extracting from the DB the required
// data for CSA, running the CSA algorithm, going back to the DB to pull a few
// extra fields for presentation, and finally formatting the data into route
// legs for easy consumption.
func RTRoute(runtime *apirtinit.MobrouteRuntime, params *RouteParams) (*RouteResponse, error) {
	utillog.Infoj("Running RTRoute against input:", params)

	// Defaults (automatically set if zero value seen)
	params.Time = fif(params.Time != nil, params.Time, ptr(time.Now())) // E.g. now
	params.MaxNTransfers = fif(params.MaxNTransfers != 0, params.MaxNTransfers, 20)
	params.MaxTransferSeconds = fif(params.MaxTransferSeconds != 0, params.MaxTransferSeconds, 40*60) // 40min
	params.MaxTripSeconds = fif(params.MaxTripSeconds != 0, params.MaxTripSeconds, 240*60)            // 240min (4hrs)
	params.MaxWalkSeconds = fif(params.MaxWalkSeconds != 0, params.MaxWalkSeconds, 20*60)             // 20min
	params.MinTransferSeconds = fif(params.MinTransferSeconds != 0, params.MinTransferSeconds, 3*60)  // 3min
	params.WalkspeedKmHr = fif(params.WalkspeedKmHr != 0.0, params.WalkspeedKmHr, 3.5)
	params.Optimizations = fif(len(params.Optimizations) != 0, params.Optimizations, []string{"feedsclean"})

	// Validations (struct validations & query bounds check from/to fields are actually in GTFS feed stops)
	if err := validator.New(validator.WithRequiredStructEnabled()).Struct(params); err != nil {
		return nil, fmt.Errorf("Validation failed for route request: %v", err)
	} else {
		utillog.Infoj("Expanded (defaultized) params RTRoute:", params)
		return rtRouteCore(runtime, params)
	}
}

func rtRouteCore(runtime *apirtinit.MobrouteRuntime, params *RouteParams) (*RouteResponse, error) {
	var (
		feedIDs              = params.FeedIDs
		timeYYYYMMDD         = utilfuncs.DayYYYYMMDD(params.Time)
		_, timeUTCOffsetSecs = params.Time.Zone()
		timeSecondsFrom      = timeUTCOffsetSecs + int(utilfuncs.DaySeconds(params.Time))
		timeSecondsTo        = timeSecondsFrom + int(params.MaxTripSeconds)
		maxKmWalkSeconds     = calcwalk.WalkKmInSecondsGivenKmHr(params.MaxWalkSeconds, params.WalkspeedKmHr)
		timer                = Timer{lastTime: time.Now()}
		diagnostics          = diagnosticst.RouteDiagnostics{}

		err                      error
		tzToUTCDeltasMap         map[string]int
		memConns                 *[]dbt.Connection
		memConnsverbose          *[]dbt.ConnectionVerbose
		memTransfers             *[]dbt.Transfer
		memWalksfrom, memWalksto *[]dbt.StopWalk
		csaConns                 []*dbt.Connection
	)

	// Essentially a 4 step process:
	//
	//  1. CSA Inputs Dataextract:
	//     Extracts necessary data from the SQLite database, 4 inputs
	//     extracted: connectionss, transfers, walksfrom, and walksto
	//
	//  2. CSA Execution:
	//     Runs the CSA algorithm (returns []*dbt.Connection)
	//
	//  3. CSA Outputs Verbosity Dataextract:
	//     Result of CSA algorithm is fitered []*dbt.Connection, which is a minimal
	//     datastructure with just information needed for CSA - so in this
	//     step we go back to the database to extract []*dbt.ConnectionVerbose
	//     for the result of the CSA alg's connections.
	//
	//  4. Formatting:
	//     Formatting the []*dbt.ConnectionVerbose CSA results into a number
	//     of formats, the primary one being 'legs' containing:
	//     - Leg for walking to input stop
	//     - (n) trips (with each []*dbt.Connection nested within)
	//     - (n) transfers between each trip
	//     - Leg for walking from output stop to destination

	// (0) Query checks
	if utilfuncs.ItemInArray("feedsclean", params.Optimizations) {
		// Optimization 'feedsclean':
		//   Limit queried feed ids to only be those that have bbox that contain
		//   either the from or to coordinate
		if fromFeedIDs, err := dbquery.QueryBoundsMatch(runtime.DB, feedIDs, params.From); err != nil {
			return nil, fmt.Errorf("From %v failed to match any bounds: %v", params.From, err)
		} else if toFeedIDs, err := dbquery.QueryBoundsMatch(runtime.DB, feedIDs, params.To); err != nil {
			return nil, fmt.Errorf("To %v failed to match any bounds: %v", params.To, err)
		} else if len(fromFeedIDs) == 0 || len(toFeedIDs) == 0 {
			return nil, fmt.Errorf("Optimization feedsclean failed to match any feeds for from/to")
		} else {
			feedIDs = utilfuncs.UniqueInts(append(fromFeedIDs, toFeedIDs...))
			diagnostics.PerfOptimizedFeeds = feedIDs
			utillog.Infof("Optimization feedsclean: Input feed IDs %v reduced to => %v", params.FeedIDs, feedIDs)
		}
	}
	if err := dbquery.QueryFeedIDsLoadedAndComputed(runtime.DB, feedIDs); err != nil {
		return nil, err
	}

	// (1.1) Memload connections
	if timezoneStrs, err := dbquery.LoadAgencyTimezones(
		runtime.DB, feedIDs,
	); err != nil {
		return nil, fmt.Errorf("Problem querying timezones from DB: %v", err)
	} else if tzToUTCDeltasMap, err = utilfuncs.TimezoneToUTCDeltaMap(*timezoneStrs, timeYYYYMMDD); err != nil {
		return nil, fmt.Errorf("Coudln't convert timezone strings to UTC deltas: %v", err)
	} else if memConns, err = dbquery.LoadConnections(
		runtime.DB, feedIDs, tzToUTCDeltasMap, timeYYYYMMDD, timeSecondsFrom, timeSecondsTo,
	); err != nil {
		return nil, fmt.Errorf("Failed to load connections from DB: %v", err)
	} else if len(*memConns) == 0 {
		return nil, fmt.Errorf(
			"Empty connections set for Feed ID %v, date %d from UTC seconds %d -> %d",
			feedIDs, timeYYYYMMDD, timeSecondsFrom, timeSecondsTo,
		)
	} else {
		diagnostics.PerfSMemloadConns = timer.Snapshot()
		diagnostics.PerfNMemConns = len(*memConns)
	}

	// (1.2) Memload transfers
	if memTransfers, err = dbquery.LoadTransfers(
		runtime.DB,
		feedIDs,
		params.MinTransferSeconds, params.MaxTransferSeconds,
		params.WalkspeedKmHr,
		params.TransferCategories,
	); err != nil {
		return nil, fmt.Errorf("Failed to load transfers from DB: %v", err)
	} else {
		diagnostics.PerfSMemloadTransfers = timer.Snapshot()
		diagnostics.PerfNMemTransfers = len(*memTransfers)
	}

	// (1.3) Memload walksfrom
	if memWalksfrom, err = dbquery.LoadStopWalks(runtime.DB, feedIDs, params.From, maxKmWalkSeconds, "", -1); err != nil {
		return nil, fmt.Errorf("Failed to load walksfrom from DB: %v", err)
	} else if nonNilWalksN := calcwalk.CountNonNilWalks(memWalksfrom); nonNilWalksN == 0 {
		return nil, fmt.Errorf("Failed to determine >0 stopwalks to from dest: %v within max walk km: %0.2f", params.From, maxKmWalkSeconds)
	} else {
		diagnostics.PerfSMemloadWalksfrom = timer.Snapshot()
		diagnostics.PerfNMemWalksfrom = len(*memWalksfrom)
		diagnostics.PerfNMemWalksfromNonnil = nonNilWalksN
	}

	// (1.4) Memload walksto
	if memWalksto, err = dbquery.LoadStopWalks(runtime.DB, feedIDs, params.To, maxKmWalkSeconds, "", -1); err != nil {
		return nil, fmt.Errorf("Failed to load walksto from DB: %v", err)
	} else if nonNilWalksN := calcwalk.CountNonNilWalks(memWalksto); nonNilWalksN == 0 {
		return nil, fmt.Errorf("Failed to determine >0 stopwalks to to dest: %v within max walk km: %0.2f", params.To, maxKmWalkSeconds)
	} else {
		diagnostics.PerfSMemloadWalksto = timer.Snapshot()
		diagnostics.PerfNMemWalksto = len(*memWalksto)
		diagnostics.PerfNMemWalkstoNonnil = nonNilWalksN
	}

	// (2) Run CSA
	if csaConns, err = csa.ExecuteCSA( // (2)
		memConns,
		memTransfers,
		calcwalk.WalksToMapTimeSecs(memWalksfrom, params.WalkspeedKmHr),
		calcwalk.WalksToMapTimeSecs(memWalksto, params.WalkspeedKmHr),
		int(timeSecondsFrom),
		params.MaxTransferSeconds,
		params.MaxWalkSeconds,
		params.MaxNTransfers,
	); err != nil {
		return nil, fmt.Errorf("Failed to run CSA: %v", err)
	} else {
		diagnostics.PerfSRoute = timer.Snapshot()
	}

	// (3)  Verbose connections
	if memConnsverbose, err = dbquery.LoadConnectionsVerbose( // (3)
		runtime.DB, feedIDs, tzToUTCDeltasMap, timeYYYYMMDD, timeSecondsFrom, timeSecondsTo,
		ptr(csa.ConnIDs(csaConns)), nil,
	); err != nil {
		return nil, err
	} else {
		diagnostics.PerfSMemloadConnsverb = timer.Snapshot()
	}

	// (4) Formatting output
	return formatRouteResponse(&diagnostics, memConnsverbose, memTransfers, params), nil
}
