package formattergeojson

import (
	"fmt"

	"git.sr.ht/~mil/mobroute/api/apirtroute/formatterlegs"
)

func FormatGeoJSON(legs *[]formatterlegs.RouteLeg, metadata string) *[]GeoJSONFeature {
	featurePoints := []GeoJSONFeature{}
	featureLineStringGeometry := GeoJSONGeometryLineString{
		Type:        "LineString",
		Coordinates: [][2]float64{},
	}

	// Generate (1) GeoJSON Line Feature to aggregate 'featureLineString'
	for legI, leg := range *legs {
		featureLineStringGeometry.Coordinates = append(
			featureLineStringGeometry.Coordinates,
			legToCoordinates(&leg, legI)...,
		)
	}

	// Generate (n+) GeoJSON Point Features
	for legI, leg := range *legs {
		featurePoints = append(
			featurePoints,
			legToGeoJSONFeaturePoints(&leg, legI)...,
		)
	}

	returnFeatures := append(
		featurePoints,
		GeoJSONFeature{
			Type: "Feature",
			Properties: map[string]string{
				"name":     "Route",
				"metadata": metadata,
			},
			Geometry: featureLineStringGeometry,
		},
	)

	return &returnFeatures
}

func legToCoordinates(leg *formatterlegs.RouteLeg, legI int) [][2]float64 {
	coords := [][2]float64{}
	coords = append(coords, latLonToLonLatCoords(leg.LegFromCoords))
	if leg.LegType == "trip" {
		for _, tripStop := range *leg.TripStops {
			coords = append(coords, latLonToLonLatCoords(tripStop.Coords))
		}
	}
	coords = append(coords, latLonToLonLatCoords(leg.LegToCoords))
	return coords
}

func legToGeoJSONFeaturePoints(leg *formatterlegs.RouteLeg, legI int) []GeoJSONFeature {
	points := []GeoJSONFeature{}

	if leg.LegType == "walk" && legI == 0 { // Initial walk from origin
		points = append(points, GeoJSONFeature{
			Type: "Feature",
			Properties: map[string]string{
				"icon": "walk",
				"name": fmt.Sprintf(
					"Walk from origin (%v): %s → %s (%s)",
					leg.LegFromCoords,
					leg.LegBeginTime.Format(FormatTime),
					leg.LegEndTime.Format(FormatTime),
					leg.LegDuration,
				),
			},
			Geometry: GeoJSONGeometryPoint{
				Type:        "Point",
				Coordinates: latLonToLonLatCoords(leg.LegFromCoords),
			},
		})
	} else if leg.LegType == "walk" && legI != 0 { // Final walk to destination
		points = append(points, GeoJSONFeature{
			Type: "Feature",
			Properties: map[string]string{
				"icon": "walk",
				"name": fmt.Sprintf(
					"Walk to destination (%v): %s → %s (%s)",
					leg.LegToCoords,
					leg.LegBeginTime.Format(FormatTime),
					leg.LegEndTime.Format(FormatTime),
					leg.LegDuration,
				),
			},
			Geometry: GeoJSONGeometryPoint{
				Type:        "Point",
				Coordinates: latLonToLonLatCoords(leg.LegToCoords),
			},
		})
	} else if leg.LegType == "trip" {
		tripHeadsign := ""
		if leg.TripRouteHeadsign != nil {
			tripHeadsign = fmt.Sprintf(" toward %s", *leg.TripRouteHeadsign)
		}
		points = append(points, GeoJSONFeature{
			Type: "Feature",
			Properties: map[string]string{
				"icon": "board",
				"name": fmt.Sprintf(
					"Board trip (%s%s) via %s to %s: %s → %s (%s)",
					*leg.TripRoute,
					tripHeadsign,
					*leg.TripFromSIDU,
					*leg.TripToSIDU,
					leg.LegBeginTime.Format(FormatTime),
					leg.LegEndTime.Format(FormatTime),
					leg.LegDuration,
				),
			},
			Geometry: GeoJSONGeometryPoint{
				Type:        "Point",
				Coordinates: latLonToLonLatCoords(leg.LegFromCoords),
			},
		})
		for i, tripStop := range *leg.TripStops {
			if i == 0 {
				continue
			}
			points = append(points, GeoJSONFeature{
				Type: "Feature",
				Properties: map[string]string{
					"icon": "stop",
					"name": fmt.Sprintf("Trip stop (%s) at %s", tripStop.Name, tripStop.Time.Format(FormatTime)),
				},
				Geometry: GeoJSONGeometryPoint{
					Type:        "Point",
					Coordinates: latLonToLonLatCoords(tripStop.Coords),
				},
			})
		}
	} else if leg.LegType == "transfer" {
		points = append(points, GeoJSONFeature{
			Type: "Feature",
			Properties: map[string]string{
				"icon": "board",
				"name": "Transfer",
			},
			Geometry: GeoJSONGeometryPoint{
				Type:        "Point",
				Coordinates: latLonToLonLatCoords(leg.LegToCoords),
			},
		})
	}

	return points
}

func latLonToLonLatCoords(latLonCoords [2]float64) [2]float64 {
	return [2]float64{latLonCoords[1], latLonCoords[0]}
}
