package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
)

var (
	subcmdRouteExamples = map[string]string{
		"Brussels Routing Example": `mobroute route -p '{"route_params":{"feed_ids":[1088],"from":[50.85728,4.351426],"to":[50.83214,4.350534],"transfer_categories":["f","i"],"output_formats":["legs","diagnostics","mapurl","request"]}}'`,
		"Krakow Routing Example":   `mobroute route -p '{"route_params":{"feed_ids":[1270],"from":[50.012338,19.88192],"to":[50.08785,20.02387],"transfer_categories":["f","i"],"output_formats":["legs","diagnostics","mapurl","request"]}}'`,
		"NYC Routing Example":      `mobroute route -p '{"route_params":{"feed_ids":[516],"from":[40.70940,-74.00537],"to":[40.72879,-73.95215],"transfer_categories":["f","i"],"output_formats":["legs","diagnostics","mapurl","request"]}}'`,
	}
)

const (
	subcmdRouteDesc = `
The route subcommand allows you to perform routing calculations
using the CSA algorithm. All functionality from loading GTFS data
(by feed id), to loading the required data to memory, to performing the
routing calculation via the CSA algorithm, and finally to formatting
the result in several formats is handled by this single CLI subcommand.
`

	subcmdRouteDocParams = `
JSON Parameters for route subcommand, should be in format of: {"route_params": {"feed_ids": [516],...}}

Required route parameters:
  - feed_ids: Array of integer feed IDs (MDBID) to use for routing requests (ex: [516, 510])
  - from: Array of two floats in lat, lon format (ex: [50.012338, 19.88192])
  - to: Array of two floats in lat, lon format (ex: [50.012338, 19.88192])
  - transfer_categories: Array of transfers categories to load - may include "f" for feed, "i" for implicit, "g" for generated (ex: ["f", "i"])
  - output_formats: Array of formats for the output may include - may include "legs", "geojson", "diagnostics", "mapurl", "request" (ex: ["legs", "geojson"])

Optional route parameters:
  - time: Datetime string at which route should depart in RFC3339 format (default: current system time)
  - max_walk_seconds: Maximum seconds walking distance on start/end of trip to get to/from origin/destination stops (default: 1200)
  - max_trip_seconds: Maximum seconds the entire route/trip can take (default: 14400)
  - min_transfer_seconds: Minimum seconds each transfer in the route can take (default: 180)
  - max_transfer_seconds: Maximum seconds each transfer in the route can take (default: 2400)
  - max_n_transfers: Maximum number of transfers a route can contain (default: 20)
  - walkspeed_km_hr: Ratio of kilometers per hour the user walk (default: 4.5)
  - optimizations: Array of optimizations to apply (default: ["feedsclean"])
`
)

func cmdRoute(flagRouteSet *flag.FlagSet, flagRouteParams *string) {
	var routeParams mobroute.OneshotRouteRequest
	flagRouteSet.Usage = usageFunc("route", flagRouteSet, subcmdRouteDesc, subcmdRouteExamples)
	flagRouteSet.Parse(os.Args[2:])
	if *flagRouteParams == "" {
		flagRouteSet.Usage()
	} else if err := json.Unmarshal([]byte(*flagRouteParams), &routeParams); err != nil {
		errorDie(os.Args[1], *flagRouteParams, err)
	} else if commandData, err := mobroute.OneshotRoute(&routeParams); err != nil {
		errorDie(os.Args[1], *flagRouteParams, err)
	} else if jsonBytes, err := json.MarshalIndent(commandData, "", "  "); err != nil {
		errorDie(os.Args[1], *flagRouteParams, err)
	} else {
		fmt.Println(string(jsonBytes))
	}
}
