package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"git.sr.ht/~mil/mobroute"
)

var (
	subcmdScheduleExamples = map[string]string{
		"Brussels Schedule Example": `mobroute schedule -p '{"schedule_params": {"feed_ids": [1088], "stop_ids": ["1088_7225"], "max_seconds": 99999}}'`,
		"Krakow Schedule Example":   `mobroute schedule -p '{"schedule_params": {"feed_ids": [1270], "stop_ids": ["1270_stop_293_61329"], "max_seconds": 99999}}'`,
		"NYC Schedule Example":      `mobroute schedule -p '{"schedule_params": {"feed_ids": [516], "stop_ids": ["516_S30"], "max_seconds": 99999}}'`,
	}
)

const (
	subcmdScheduleDesc = `
The schedule subcommand allows you to query departure schedules for a
given set of stop_ids for the given feeds given a input date and maximum
seconds delta threshold.  The underlying logic for timeconversion and
timetable assembly is the same between the schedule and routing logic
so this may also be used as a helpful diagnostic tool.

The stop_ids provided should be in the format of FEEDID_STOPID to
disambiguate between feeds with duplicate stop ID handles. Stop IDs can
be queried using the stops command.
	`
	subcmdScheduleDocParams = `
JSON Parameters for schedule subcommand, should be in format of: {"schedule_params": {"feed_ids": [516],...}}

Required schedule parameters:
  - feed_ids: Array of integer feed IDs (MDBID) to use for routing requests (ex: [1088, 516])
  - stop_ids: Array of string stop_ids in FEEDID_RAWSTOPID format (ex: ["1088_7225", "516_S30"])
  - max_seconds: Cutoff threshold in integer seconds from departure time to consider (ex: 600)

Optional schedule parameters:
  - time: Datetime string at which to consider departures from in RFC3339 format (default: current system time)
`
)

func cmdSchedule(flagScheduleSet *flag.FlagSet, flagScheduleParams *string) {
	var scheduleParams mobroute.OneshotScheduleRequest
	flagScheduleSet.Usage = usageFunc("schedule", flagScheduleSet, subcmdScheduleDesc, subcmdScheduleExamples)
	flagScheduleSet.Parse(os.Args[2:])
	if *flagScheduleParams == "" {
		flagScheduleSet.Usage()
	} else if err := json.Unmarshal([]byte(*flagScheduleParams), &scheduleParams); err != nil {
		errorDie(os.Args[1], *flagScheduleParams, err)
	} else if commandData, err := mobroute.OneshotSchedule(&scheduleParams); err != nil {
		errorDie(os.Args[1], *flagScheduleParams, err)
	} else if jsonBytes, err := json.MarshalIndent(commandData, "", "  "); err != nil {
		errorDie(os.Args[1], *flagScheduleParams, err)
	} else {
		fmt.Println(string(jsonBytes))
	}
}
