package csa_test

import (
	"git.sr.ht/~mil/mobroute/csa"
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utillog"
	"testing"
)

type CheapestArrivalDestinationTestCaseArgs struct {
	StopToArrivalDetails map[string]csa.ArrivalDetails
	StopWalkcosts        map[string]uint
	MaxWalkcost          uint
}

type CheapestArrivalDestinationTestCase struct {
	Test   string
	Args   CheapestArrivalDestinationTestCaseArgs
	Expect string
}

func Test_CheapestArrivalDestination(t *testing.T) {
	utillog.LogWarn = true

	var (
		testCases = []CheapestArrivalDestinationTestCase{
			{
				Test: "Determines cheapest arrival destination (b)",
				Args: CheapestArrivalDestinationTestCaseArgs{
					StopToArrivalDetails: map[string]csa.ArrivalDetails{
						"a": csa.ArrivalDetails{Time: 7, Connection: &dbt.Connection{}},
						"b": csa.ArrivalDetails{Time: 2, Connection: &dbt.Connection{}},
						"c": csa.ArrivalDetails{Time: 5, Connection: &dbt.Connection{}},
						"d": csa.ArrivalDetails{Time: 8, Connection: &dbt.Connection{}},
					},
					StopWalkcosts: map[string]uint{"a": 1, "b": 2, "c": 3, "d": 4},
					MaxWalkcost:   99,
				},
				Expect: "b", // E.g. cheapest as b (2 + 2 = 4) less then all other options
			},
			{
				Test: "Determines cheapest arrival destination (c)",
				Args: CheapestArrivalDestinationTestCaseArgs{
					StopToArrivalDetails: map[string]csa.ArrivalDetails{
						"a": csa.ArrivalDetails{Time: 7, Connection: &dbt.Connection{}},
						"b": csa.ArrivalDetails{Time: 6, Connection: &dbt.Connection{}},
						"c": csa.ArrivalDetails{Time: 3, Connection: &dbt.Connection{}},
						"d": csa.ArrivalDetails{Time: 8, Connection: &dbt.Connection{}},
					},
					StopWalkcosts: map[string]uint{"a": 3, "b": 9, "c": 6, "d": 4},
					MaxWalkcost:   99,
				},
				Expect: "c", // E.g. cheapest as c (3 + 6 = 9) less then all other options
			},
			{
				Test: "Determines cheapest arrival destination properly respecting max walkcost",
				Args: CheapestArrivalDestinationTestCaseArgs{
					StopToArrivalDetails: map[string]csa.ArrivalDetails{
						"a": csa.ArrivalDetails{Time: 7, Connection: &dbt.Connection{}},
						"b": csa.ArrivalDetails{Time: 6, Connection: &dbt.Connection{}},
						"c": csa.ArrivalDetails{Time: 3, Connection: &dbt.Connection{}},
						"d": csa.ArrivalDetails{Time: 8, Connection: &dbt.Connection{}},
					},
					StopWalkcosts: map[string]uint{"a": 3, "b": 9, "c": 6, "d": 4},
					MaxWalkcost:   5,
				},
				Expect: "a", // E.g. cheapest as a (7 + 3 = 10).. ignores c since > max walk cost
			},
			// TODO: unit test to ignore entries where connection is nil
		}
	)

	for _, testCase := range testCases {
		if result, err := csa.CheapestArrivalDestination(
			testCase.Args.StopToArrivalDetails,
			testCase.Args.StopWalkcosts,
			testCase.Args.MaxWalkcost,
		); err != nil {
			t.Fatalf("CheapestArrivalDestination error: %v", err)
		} else if result != testCase.Expect {
			t.Fatalf("Want %v; got %v", testCase.Expect, result)
		}
	}
}
