-- Lookups up the 'ultimate_parent' for the given stop_id feed_id combination
-- In all cases except for location_type=4 this is directly accessible in the
-- row itself. For location_type=4, there is a join based on platform which
-- may trigger '1-hop'.
--
-- This was modeled directly off of GTFS spec based on the logic defined
-- in stops.txt parent_station:

-- Defines hierarchy between the different locations defined in stops.txt.
-- It contains the ID of the parent location, as followed:
--   - Stop/platform (location_type=0): the parent_station field contains the ID of a station.
--   - Station (location_type=1): this field must be empty.
--   - Entrance/exit (location_type=2) or generic node (location_type=3): the parent_station field contains the ID of a station (location_type=1)
--   - Boarding Area (location_type=4): the parent_station field contains ID of a platform.
--
-- Conditionally Required:
--   - Required for locations which are entrances (location_type=2), generic nodes (location_type=3) or boarding areas (location_type=4).
--   - Optional for stops/platforms (location_type=0).
--   - Forbidden for stations (location_type=1).
--
with
rootstations as (
  -- Root Stations (location_type=1)
  select s.feed_id, s.stop_id, s.stop_id as ultimate_parent
  from stops s
  where s.location_type = 1
),
eenodes as (
  -- Entrance/exits/nodes (location_type=2,3)
  -- 1-level lookup: node->station
  select s.feed_id, s.stop_id, s.parent_station as ultimate_parent
  from stops s
  where s.location_type = 2 or s.location_type = 3
),
platforms as (
  -- Stops/platforms (location_type=0)
  -- 0-level lookup: boarding_area->boarding_area (NO parent_station)
  -- 1-level lookup: boarding_area->parent_station
  select s.feed_id, s.stop_id, coalesce(s.parent_station, s.stop_id) as ultimate_parent
  from stops s
  where s.location_type = 0 or s.location_type is null
),
boardingareas as (
  -- Boarding areas (location_type=4)
  -- 1-level lookup: boarding_area->platform OR
  -- 2-level lookup: boarding_area->platform->station
  select s.feed_id, s.stop_id, coalesce(p.ultimate_parent, p.stop_id) as ultimate_parent
  from stops s join platforms p on s.feed_id = p.feed_id and s.parent_station = p.stop_id
  where s.location_type = 4
)
select feed_id, stop_id, ultimate_parent as parent_station from rootstations
union all 
select feed_id, stop_id, ultimate_parent as parent_station from eenodes
union all 
select feed_id, stop_id, ultimate_parent as parent_station from platforms
union all
select feed_id, stop_id, ultimate_parent as parent_station from boardingareas
