package testhelper

import (
	"archive/zip"
	"io/ioutil"
	"os"
	"path/filepath"

	"gopkg.in/yaml.v3"
)

// YMLToCSVZip takes a YML filepath as input with YML sepcification for a ZIP
// of CSVs looking like:
//
// foo.csv:
//   - { bar: baz, bil: bin }
//   - { bar: baz, bil: bin }
//
// bar.csv:
//   - { bar: baz, bil: bin }
//   - { bar: baz, bil: bin }
//
// And writes out a ZIP archive with the corresponding files of CSVs.
// For example in the above sample, 2 CSV files would be in the ZIP (foo.csv,
// and bar.csv) each with 2 rows.
func YMLToCSVZip(yamlFilepath string) (string, error) {
	var (
		csvFolder string
		zipFile   *os.File
		err       error
		zipWriter *zip.Writer
		data      map[string][]map[string]any
	)

	// Create tempdir & ZIP archive
	if csvFolder, err = ioutil.TempDir("", "ymlcsvzip"); err != nil {
		return "", err
	} else if zipFile, err = os.CreateTemp("", "*.zip"); err != nil {
		return "", err
	} else {
		zipWriter = zip.NewWriter(zipFile)
	}
	defer os.RemoveAll(csvFolder)
	defer zipFile.Close()
	defer zipWriter.Close()

	// Read YML
	if ymlContent, err := ioutil.ReadFile(yamlFilepath); err != nil {
		return "", err
	} else if err := yaml.Unmarshal(ymlContent, &data); err != nil {
		return "", err
	}

	// Loop each key in YML, write file to zip archive with contents converted
	// to CSV
	for fileName, records := range data {
		csvFilepath := filepath.Join(csvFolder, fileName)

		// Copy CSV to Zip File
		if err := ArrayOfMapsToCSV(records, csvFilepath); err != nil {
			return "", err
		} else if relPath, err := filepath.Rel(csvFolder, csvFilepath); err != nil {
			return "", err
		} else if zipEntry, err := zipWriter.Create(relPath); err != nil {
			return "", err
		} else if fileContent, err := ioutil.ReadFile(csvFilepath); err != nil {
			return "", err
		} else {
			zipEntry.Write(fileContent)
		}
	}
	return zipFile.Name(), nil
}
